from __future__ import print_function
from past.builtins import xrange

import zlib

from vstruct import VStruct,VArray
from vstruct.bitfield import *
from vstruct.primitives import *

'''
0 End 
1 ShowFrame 
2 DefineShape 
4 PlaceObject 
5 RemoveObject 
6 DefineBits 
7 DefineButton 
8 JPEGTables 
9 SetBackgroundColor 
10 DefineFont 
11 DefineText 
12 DoAction 
13 DefineFontInfo 
14 DefineSound 
15 StartSound 
17 DefineButtonSound 
18 SoundStreamHead 
19 SoundStreamBlock 
20 DefineBitsLossless 
21 DefineBitsJPEG2 
22 DefineShape2 
23 DefineButtonCxform
24 Protect
26 PlaceObject2 
28 RemoveObject2 
32 DefineShape3 
33 DefineText2 
34 DefineButton2 
35 DefineBitsJPEG3 
36 DefineBitsLossless2 
37 DefineEditText 
39 DefineSprite 
43 FrameLabel 
45 SoundStreamHead2 
46 DefineMorphShape 
48 DefineFont2 
56 ExportAssets 
57 ImportAssets 
58 EnableDebugger 
59 DoInitAction 
60 DefineVideoStream 
61 VideoFrame 
62 DefineFontInfo2 
64 EnableDebugger2 
65 ScriptLimits 
66 SetTabIndex 
69 FileAttributes
70 PlaceObject3
71 ImportAssets2 
73 DefineFontAlignZones 
74 CSMTextSettings 
75 DefineFont3 
76 SymbolClass 
77 Metadata 
78 DefineScalingGrid 
82 DoABC 
83 DefineShape4 
84 DefineMorphShape2 
86 DefineSceneAndFrameLabelData 
87 DefineBinaryData 
88 DefineFontName 
89 StartSound2 
90 DefineBitsJPEG4 
91 DefineFont4 
93 EnableTelemetry 
'''

swftags = {
    0:'End',
    1:'ShowFrame',
    2:'DefineShape',
    4:'PlaceObject',
    5:'RemoveObject',
    6:'DefineBits',
    7:'DefineButton',
    8:'JPEGTables',
    9:'SetBackgroundColor',
    10:'DefineFont',
    11:'DefineText',
    12:'DoAction',
    13:'DefineFontInfo',
    14:'DefineSound',
    15:'StartSound',
    17:'DefineButtonSound',
    18:'SoundStreamHead',
    19:'SoundStreamBlock',
    20:'DefineBitsLossless',
    21:'DefineBitsJPEG2',
    22:'DefineShape2',
    23:'DefineButtonCxform',
    24:'Protect',
    26:'PlaceObject2',
    28:'RemoveObject2',
    32:'DefineShape3',
    33:'DefineText2',
    34:'DefineButton2',
    35:'DefineBitsJPEG3',
    36:'DefineBitsLossless2',
    37:'DefineEditText',
    39:'DefineSprite',
    41:'SerialNumber',
    43:'FrameLabel',
    45:'SoundStreamHead2',
    46:'DefineMorphShape',
    48:'DefineFont2',
    56:'ExportAssets',
    57:'ImportAssets',
    58:'EnableDebugger',
    59:'DoInitAction',
    60:'DefineVideoStream',
    61:'VideoFrame',
    62:'DefineFontInfo2',
    64:'EnableDebugger2',
    65:'ScriptLimits',
    66:'SetTabIndex',
    69:'FileAttributes',
    70:'PlaceObject3',
    71:'ImportAssets2',
    73:'DefineFontAlignZones',
    74:'CSMTextSettings',
    75:'DefineFont3',
    76:'SymbolClass',
    77:'Metadata',
    78:'DefineScalingGrid',
    82:'DoABC',
    83:'DefineShape4',
    84:'DefineMorphShape2',
    86:'DefineSceneAndFrameLabelData',
    87:'DefineBinaryData',
    88:'DefineFontName',
    89:'StartSound2',
    90:'DefineBitsJPEG4',
    91:'DefineFont4',
    93:'EnableTelemetry',
}


class RECT(VBitField):
    def __init__(self):
        VBitField.__init__(self)
        self.Nbits  = v_bits(5)
        self.Xmin   = v_bits(1)
        self.Xmax   = v_bits(1)
        self.Ymin   = v_bits(1)
        self.Ymax   = v_bits(1)

    def pcb_Nbits(self):
        bitwidth = self.Nbits
        self['Xmin'].vsSetBitWidth(bitwidth)
        self['Xmax'].vsSetBitWidth(bitwidth)
        self['Ymin'].vsSetBitWidth(bitwidth)
        self['Ymax'].vsSetBitWidth(bitwidth)

class EncodedU32(v_number):

    def __init__(self):
        v_number.__init__(self)
        self._vs_fmt = None # to prevent misuse

    def vsParse(self, bytez, offset=0):
        self._vs_value = 0
        # this shit is *not* my fault... ;)
        for i in xrange(5):
            b = bytez[offset + i]
            self._vs_value |= (b & 0x7f) << (7 * i)
            if not b & 0x80:
                break
        self._vs_length = i + 1
        return offset + self._vs_length

    def vsEmit(self):
        raise Exception('swf.EncodedU32() needs vsEmit!')

    def __repr__(self):
        return 'U32: %d' % self.vsGetValue()


class RGB(VStruct):
    def __init__(self):
        VStruct.__init__(self)
        self.Red    = v_uint8()
        self.Green  = v_uint8()
        self.Blue   = v_uint8()

class RGBA(VStruct):
    def __init__(self):
        VStruct.__init__(self)
        self.Red    = v_uint8()
        self.Green  = v_uint8()
        self.Blue   = v_uint8()
        self.Alpha  = v_uint8()

class MATRIX(VBitField):
    def __init__(self):
        VBitField.__init__(self)
        self.HasScale = v_bits(1)
        self.NScaleBits  = v_bits(5)
        self.ScaleX      = v_bits(1)
        self.ScaleY      = v_bits(1)
        self.HasRotate   = v_bits(1)
        self.NRotateBits = v_bits(5)
        self.RotateSkew0 = v_bits(1)
        self.RotateSkew1 = v_bits(1)
        self.NTransBits  = v_bits(5)
        self.TranslateX  = v_bits(1)
        self.TranslateY  = v_bits(1)

    def pcb_HasScale(self):
        if not self.HasScale:
            self.vsDelField('NScaleBits')
            self.vsDelField('ScaleX')
            self.vsDelField('ScaleY')

    def pcb_NScaleBits(self):
        bitwidth = self.NScaleBits
        self['ScaleX'].vsSetBitWidth(bitwidth)
        self['ScaleY'].vsSetBitWidth(bitwidth)

    def pcb_HasRotate(self):
        if not self.HasRotate:
            self.vsDelField('NRotateBits')
            self.vsDelField('RotateSkew0')
            self.vsDelField('RotateSkew1')

    def pcb_NRotateBits(self):
        bitwidth = self.NRotateBits
        self['RotateSkew0'].vsSetBitWidth(bitwidth)
        self['RotateSkew1'].vsSetBitWidth(bitwidth)

    def pcb_NTransBits(self):
        bitwidth = self.NTransBits
        self['TranslateX'].vsSetBitWidth(bitwidth)
        self['TranslateY'].vsSetBitWidth(bitwidth)

class GRADRECORD(VStruct):
    def __init__(self,shape=1):
        VStruct.__init__(self)
        self.Ratio  = v_uint8()
        if shape <= 2:
            self.Color = RGB()
        else:
            self.Color = RGBA()

class GRADIENT(VStruct):
    def __init__(self,shape=1):
        VStruct.__init__(self)
        self._swf_shape = shape
        self.BitFields       = VBitField()
        self.BitFields.SpreadMode           = v_bits(2)
        self.BitFields.InterpolationMode    = v_bits(2)
        self.BitFields.NumGradients         = v_bits(4)
        self.GradientRecords = VArray()

    def pcb_BitFields(self):
        gcount = self.BitFields.NumGradients
        for i in xrange(gcount):
            g = GRADRECORD(shape=self._swf_shape)
            self.GradientRecords.vsAddElement(g)

class FOCALGRADIENT(GRADIENT):
    def __init__(self,shape=4):
        GRADIENT.__init__(self)
        self.FocalPoint = FIXED8()

FILL_SOLID              = 0x00
FILL_LINEAR_GRADIENT    = 0x10
FILL_RADIAL_GRADIENT    = 0x12
FILL_FOCAL_GRADIENT     = 0x13
FILL_BITMAP_REPEAT      = 0x40
FILL_BITMAP_CLIPPED     = 0x41
FILL_BITMAP_NSMOOTH_REP = 0x42
FILL_BITMAP_NSMOOTH_CLP = 0x43

class FILLSTYLE(VStruct):
    def __init__(self, shape=1):
        VStruct.__init__(self)
        self._swf_shape = shape
        self.FillStyleType      = v_uint8()
        self.Color              = RGB()
        self.GradientMatrix     = MATRIX()
        self.Gradient           = GRADIENT(shape=shape)
        self.BitmapId           = v_uint16()
        self.BitmapMatrix       = MATRIX()

    def pcb_FillStyleType(self):
        fstype = self.FillStyleType
        if fstype != 0x00:
            self.vsDelField('Color')

        if fstype not in (0x10, 0x12, 0x13):
            self.vsDelField('GradientMatrix')
            self.vsDelField('Gradient')
        else:
            if fstype == 0x13:
                self.Gradient = FOCALGRADIENT()

        if fstype not in (0x40,0x41,0x42,0x43):
            self.vsDelField('BitmapId')
            self.vsDelField('BitmapMatrix')

class FILLSTYLEARRAY(VStruct):
    def __init__(self, shape=1):
        VStruct.__init__(self)
        self._swf_shape = shape
        self.FillStyleCount     = v_uint8()
        self.FillStyleCountEx   = v_uint16()
        self.FillStyles         = VArray()

    def pcb_FillStyleCount(self):
        if self.FillStyleCount != 0xff:
            self.vsDelField('FillStyleCountEx')
            self.swfAddFillStyles(self.FillStyleCount)

    def pcb_FillStyleCountEx(self):
        self.swfAddFillStyles(self.FillStyleCountEx)

    def swfAddFillStyles(self, count):
        for i in xrange(count):
            elem = FILLSTYLE(shape=self._swf_shape)
            self.FillStyles.vsAddElement(elem)

class LINESTYLE(VStruct):
    def __init__(self,shape=1):
        VStruct.__init__(self)
        self._swf_shape = shape
        self.Width  = v_uint16()
        colorcls = RGB
        if shape >= 3:
            colorcls = RGBA
        self.Color = colorcls()

class LINESTYLEARRAY(VStruct):
    def __init__(self,shape=1):
        VStruct.__init__(self)
        self._swf_shape = 1
        self.LineStyleCount     = v_uint8()
        self.LineStyleCountEx   = v_uint16()
        self.LineStyles         = VArray()

    def pcb_LineStyleCount(self):
        if self.LineStyleCount != 0xff:
            self.vsDelField('LineStyleCountEx')
            self.swfAddLineStyles(self.LineStyleCount)

    def pcb_LineStyleCountEx(self):
        self.swfAddLineStyles(self.LineStyleCountEx)

    def swfAddLineStyles(self, count):
        for i in xrange(count):
            elem = LINESTYLE(shape=self._swf_shape)
            self.LineStyles.vsAddElement(elem)

class SHAPEWITHSTYLE(VStruct):
    def __init__(self, shape=1):
        VStruct.__init__(self)
        self.FillStyles     = FILLSTYLEARRAY(shape=shape)
        self.LineStyles     = LINESTYLEARRAY(shape=shape)
        self.NumBits        = VBitField()
        self.NumBits.NumFillBits    = v_bits(4)
        self.NumBits.NumLineBits    = v_bits(4)
        self.ShapeRecords           = VArray()

class SwfColorTransform(VBitField):
    def __init__(self):
        VBitField.__init__(self)
        self.HasAddTerms    = v_bits(1)
        self.HasMultTerms   = v_bits(1)
        self.Nbits          = v_bits(4)
        self.RedMultTerm    = v_bits(1)
        self.GreenMultTerm  = v_bits(1)
        self.BlueMultTerm   = v_bits(1)
        self.RedAddTerm     = v_bits(1)
        self.GreenAddTerm   = v_bits(1)
        self.BlueAddTerm    = v_bits(1)

    def pcb_Nbits(self):
        bitwidth = self.Nbits

        if not self.HasAddTerms:
            self.vsDelField('RedAddTerm')
            self.vsDelField('GreenAddTerm')
            self.vsDelField('BlueAddTerm')
        else:
            self['RedAddTerm'].vsSetBitWidth(bitwidth)
            self['GreenAddTerm'].vsSetBitWidth(bitwidth)
            self['BlueAddTerm'].vsSetBitWidth(bitwidth)

        if not self.HasMultTerms:
            self.vsDelField('RedMultTerm')
            self.vsDelField('GreenMultTerm')
            self.vsDelField('BlueMultTerm')
        else:
            self['RedMultTerm'].vsSetBitWidth(bitwidth)
            self['GreenMultTerm'].vsSetBitWidth(bitwidth)
            self['BlueMultTerm'].vsSetBitWidth(bitwidth)

class SwfTagCodeLength(VBitField):
    def __init__(self):
        VBitField.__init__(self)
        self.TagCode = v_bits(10)
        self.TagLength = v_bits(6)

class SwfTagHeader(VStruct):
    def __init__(self):
        VStruct.__init__(self)
        self.TagCodeAndLength = v_uint16()
        self.OptionalLength   = v_uint32()

    def __repr__(self):
        tagcode = self.getSwfTagCode()
        taglen = self.getSwfTagLength()
        tagname = swftags.get(tagcode,'Unknown')
        return '%s (%d)' % (tagname,taglen)

    def pcb_TagCodeAndLength(self):
        if self.TagCodeAndLength & 0x3f != 0x3f:
            self.vsDelField('OptionalLength')

    def getSwfTagCode(self):
        return self.TagCodeAndLength >> 6

    def getSwfTagLength(self):
        taglen = self.TagCodeAndLength & 0x3f
        if taglen == 0x3f:
            taglen = self.OptionalLength
        return taglen

class PlaceObject(VStruct):
    def __init__(self, size):
        VStruct.__init__(self)
        self.CharacterId    = v_uint16()
        self.Depth          = v_uint16()
        self.Matrix         = MATRIX()
        self.ColorTransform = SwfColorTransform()

        self._swf_tagsize = size

    def vsParse(self, bytez, offset=0):
        # we *only* do this this way because we dont know
        # how big the Matrix field is until we parse it...
        offset = self['CharacterId'].vsParse(bytez,offset=offset)
        offset = self['Depth'].vsParse(bytez,offset=offset)
        offset = self['Matrix'].vsParse(bytez,offset=offset)
        if offset >= self._swf_tagsize:
            self.vsDelField('ColorTransform')
        else:
            offset = self['ColorTransform'].vsParse(bytez,offset=offset)
        return offset

class FileAttributes(VBitField):
    def __init__(self, size):
        VBitField.__init__(self)
        self.Reserved0      = v_bits(1)
        self.UseDirectBlit  = v_bits(1)
        self.UseGPU         = v_bits(1)
        self.HasMetadata    = v_bits(1)
        self.ActionScript3  = v_bits(1)
        self.Reserved1      = v_bits(2)
        self.UseNetwork     = v_bits(1)
        self.ReservedEnd    = v_bits(24)

class SetBackgroundColor(VStruct):
    def __init__(self, size):
        VStruct.__init__(self)
        self.BackgroundColor = RGB()

class OffsetName(VStruct):
    def __init__(self):
        VStruct.__init__(self)
        self.Offset     = EncodedU32()
        self.Name       = v_zstr()

class DefineSceneAndFrameLabelData(VStruct):
    def __init__(self, size):
        VStruct.__init__(self)
        self.SceneCount         = EncodedU32()
        self.Scenes             = VArray()
        self.FrameLabelCount    = EncodedU32()
        self.Frames             = VArray()

    def pcb_SceneCount(self):
        count = self.SceneCount
        self.Scenes.vsAddElements(count, OffsetName)

    def pcb_FrameLabelCount(self):
        count = self.FrameLabelCount
        self.Scenes.vsAddElements(count, OffsetName)

class DefineShape4(VStruct):
    def __init__(self, size):
        VStruct.__init__(self)
        self.ShapeId        = v_uint16()
        self.ShapeBounds    = RECT()
        self.EdgeBounds     = RECT()
        self.ShapeFlags     = VBitField()
        self.Shapes         = SHAPEWITHSTYLE(shape=4)

        self.ShapeFlags.Reserved                = v_bits(5)
        self.ShapeFlags.UsesFillWindingRule     = v_bits(1)
        self.ShapeFlags.UsesNonScalingStrokes   = v_bits(1)
        self.ShapeFlags.UsesScalingStrokes      = v_bits(1)

class SwfTag(VStruct):
    def __init__(self):
        VStruct.__init__(self)
        self.TagHeader = SwfTagHeader()
        self.TagData = VStruct()

    def pcb_TagHeader(self):
        taglen = self.TagHeader.getSwfTagLength()
        tagcode = self.TagHeader.getSwfTagCode()
        tagname = swftags.get(tagcode,'UnknownTag')
        tagclass = globals().get(tagname,v_bytes)
        self.TagData = tagclass(taglen)

class SwfHeader(VStruct):
    def __init__(self):
        VStruct.__init__(self)
        self.Signature  = v_str(3)
        self.Version    = v_uint8()
        self.FileLength = v_uint32()
        self.FrameSize  = RECT()
        self.FrameRate  = v_uint16()
        self.FrameCount = v_uint16()

class SwfFile(VStruct):

    def __init__(self):
        VStruct.__init__(self)
        self.Header = SwfHeader()
        self.Tags = VArray()

    def vsParse(self, bytez, offset=0):
        magic = bytez[offset:offset+3]
        if magic not in ('FWS','CWS','ZWS'):
            raise Exception('Invalid Swf Magic')

        if magic == 'CWS':
            bytez = bytez[:8] + zlib.decompress(bytez[8:])

        offset = self.Header.vsParse(bytez, offset=offset)
        print('Header Says: %d Bytes Are: %d' %  (self.Header.FileLength,len(bytez)))
        i = 0
        while offset < len(bytez):
            swftag = SwfTag()
            offset = swftag.vsParse(bytez, offset=offset)
            self.Tags.vsAddElement(swftag)
            i += 1
            if i > 3:
                break




'''
Parser objects for the Intel Hex file format.
'''
from __future__ import print_function
from io import open

import vstruct
from vstruct.primitives import *

IHEX_REC_DATA           = 0
IHEX_REC_EOF            = 1
IHEX_REC_EXSEG          = 2 # Extended Segment Address Records
IHEX_REC_STARTSEG       = 3 # The beginning code segment value
IHEX_REC_EXLINADDR      = 4 # Extended Linear Address Records
IHEX_REC_STARTLINADDR   = 5 # 32bit entry point

class IHexChunk(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.startcode      = v_bytes(1)
        self.bytecount      = v_bytes(2)
        self.address        = v_bytes(4)
        self.recordtype     = v_bytes(2)
        self.data           = v_bytes(0)
        self.csum           = v_bytes(2)

    def pcb_bytecount(self):
        dsize = int(self.bytecount, 16)
        self.vsGetField('data').vsSetLength( 2 * dsize )

    def getAddress(self):
        '''
        Parse the address field and return the int type.
        '''
        return int( self.address, 16 )

    def getData(self):
        '''
        Return the binary data payload for this chunk.
        '''
        return self.data.decode('hex')

class IHexFile(vstruct.VArray):

    def vsParse(self, bytes, offset=0):

        lines = bytes[offset:].splitlines()

        for line in lines:
            offset += 1 # there is an eaten newline for each
            if not line:
                continue

            c = IHexChunk()
            c.vsParse( line )
            offset += len( c )

            self.vsAddElement( c )

            if int(c.recordtype, 16) == IHEX_REC_EOF:
                break

        return offset

    def getEntryPoint(self):
        '''
        If a 32bit linear start address is defined for this file,
        return it.  Returns None if the 32bit entry point extension
        is not present.
        '''
        for fname, chunk in self:
            ctype = int( chunk.recordtype, 16 )
            if ctype == IHEX_REC_STARTLINADDR:
                return int( chunk.data, 16 )

    def getMemoryMaps(self):
        '''
        Retrieve a set of memory maps defined by this hex file.

        Memory maps are returned as a list of
        ( va, perms, fname, bytes ) tuples.
        '''
        import envi.memory as e_mem

        # Get all the binary parts....
        baseaddr = 0
        memparts = []

        for fname, chunk in self:

            ctype = int( chunk.recordtype, 16 )

            if ctype == IHEX_REC_DATA:
                addr = chunk.getAddress() + baseaddr
                memparts.append( (addr, chunk.getData()) )
                continue

            if ctype == IHEX_REC_EXSEG:
                baseaddr = int( chunk.data, 16 ) << 4
                continue

            if ctype == IHEX_REC_EXLINADDR:
                baseaddr = int( chunk.data, 16 ) << 16
                continue

            if ctype == IHEX_REC_EOF:
                break

            raise Exception('Unhandled IHEX chunk: %s' % chunk.recordtype)

        memparts.sort()

        maps = []
        for addr, bytes in memparts:
            # is this the next contiguous chunk?
            if maps and addr == ( maps[-1][0] + len(maps[-1][3]) ):
                maps[-1][3] += bytes
            else:
                maps.append( [ addr, e_mem.MM_RWX, '', bytes ] )

        return maps

if __name__ == '__main__':

    example = '''
:020000021000EC
:10C20000E0A5E6F6FDFFE0AEE00FE6FCFDFFE6FD93
:10C21000FFFFF6F50EFE4B66F2FA0CFEF2F40EFE90
:10C22000F04EF05FF06CF07DCA0050C2F086F097DF
:10C23000F04AF054BCF5204830592D02E018BB03F9
:020000020000FC
:04000000FA00000200
:00000001FF
'''
    example1 = '''
:10001300AC12AD13AE10AF1112002F8E0E8F0F2244
:10000300E50B250DF509E50A350CF5081200132259
:03000000020023D8
:0C002300787FE4F6D8FD7581130200031D
:10002F00EFF88DF0A4FFEDC5F0CEA42EFEEC88F016
:04003F00A42EFE22CB
:00000001FF
'''
    example2 = '''
:10010000214601360121470136007EFE09D2190140
:100110002146017EB7C20001FF5F16002148011988
:10012000194E79234623965778239EDA3F01B2CAA7
:100130003F0156702B5E712B722B732146013421C7
:00000001FF
asdf
    '''

    example3 = '''
:100000003C932014BBE0AD7A3EAC4D261FB267A4F2
:100010008121F4C2D641A503B6038C9932A36EBCEC
:10002000D204306AE84404FCE8C7452DE0BE3160E4
:100030005CC6E94D3F4E62765AC237EAD3C2895157
:0200000400F00A
:10000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00
:020000040030CA
:1000000000270F0F0083F500FFC0FFE0FF400000C5
:020000040020DA
:080000000102030405060708D4
:00000001FF
'''

    import sys
    h = IHexFile()
    h.vsParse( open(sys.argv[1], 'rb').read() )
    #print(h.tree())

    for addr,perms,fname,bytes in h.getMemoryMaps():
        print('0x%.8x: %r' % ( addr, bytes ))

    print(h.getEntryPoint())



import traceback

#from cancat.j1939 import *
# we can move things into here if we decide this replaces the exiting j1939 modules
import cancat
import struct
from J1939db import *
from cancat import *
from cancat.vstruct.bitfield import *

import Queue
import threading
''' 
This is a J1939 Stack module.
It's purpose is to provide a J1939-capable object which extends (consumes) CanCat's CanInterface module, and provides a J1939 interface.
In the original J1939 module, extended messages were treated like oddities.  This module will work on the premise that all messages (except TP messages) are created equal, and are available in the same queue.  TP messages will be the "specialty" messages which create new, arbitrary sized messages from smaller ones.  If they don't make it through, they don't.  This module is intended to make J1939 attack clients easier to work with, whereas the first module was focused on reverse engineering.  Let's see if we can't come up with something awesome, then possible merge them together in the future.

This module focuses around PGNs.  All messages are handled and sorted by their PS/DA.
'''

J1939MSGS = 1939

PF_RQST =       0xea
PF_TP_DT =      0xeb
PF_TP_CM =      0xec
PF_ADDRCLAIM =  0xee
PF_PROPRIETRY=  0xef
PF_KWP1 =       0xdb
PF_KWP2 =       0xda
PF_KWP3 =       0xce
PF_KWP4 =       0xcd

CM_RTS   =       0x10
CM_CTS   =       0x11
CM_EOM   =       0x13
CM_ABORT =       0xff
CM_BAM   =       0x20

TP_BAM = 20
TP_DIRECT = 10
TP_DIRECT_BROKEN=9

class NAME(VBitField):
    def __init__(self):
        VBitField.__init__(self)
        self.arbaddrcap = v_bits(1)
        self.ind_group = v_bits(3)
        self.vehicle_system_instance = v_bits(4)
        self.vehicle_system = v_bits(7)
        self.reserved = v_bits(1)
        self.function = v_bits(8)
        self.function_instance = v_bits(5)
        self.ecu_instance = v_bits(3)
        self.mfg_code = v_bits(11)
        self.identity_number = v_bits(21)

    def minrepr(self):
        mfgname = mfg_lookup.get(self.mfg_code)
        return "id: 0x%x mfg: %s" % (self.identity_number, mfgname)


def parseName(name):
    namebits= NAME()
    rname = name[::-1]
    namebits.vsParse(rname)
    return namebits

def parseArbid(arbid):
    (prioPlus,
        pf,
        ps,
        sa) = struct.unpack('BBBB', struct.pack(">I", arbid))

    prio = prioPlus >> 2
    edp = (prioPlus >> 1) & 1
    dp = prioPlus & 1

    return prio, edp, dp, pf, ps, sa


def meldExtMsgs(msgs):
    out = []
    length = msgs.get('totsize')
    for arbtup, msg in msgs.get('msgs'):
        out.append(msg[1:])

    outval = ''.join(out)
    outval = outval[:length]

    return outval

### renderers for specific PF numbers
def pf_c9(idx, ts, arbtup, data, j1939):
    b4 = data[3]
    req = "%.2x %.2x %.2x" % ([ord(d) for d in data[:3]])
    usexferpfn = ('', 'Use_Transfer_PGN', 'undef', 'NA')[b4 & 3]
    
    return "Request2: %s %s" % (req,  usexferpgn)

def pf_ea(idx, ts, (prio, edp, dp, pf, ps, sa), data, j1939):
    return "Request: %s" % (data[:3].encode('hex'))

# no pf_eb or pf_ec since those are handled at a lower-level in this stack

def pf_ee(idx, ts, (prio, edp, dp, pf, ps, sa), data, j1939):
    if ps == 255 and sa == 254:
        return 'CANNOT CLAIM ADDRESS'
    
    addrinfo = parseName(data).minrepr()
    return "Address Claim: %s" % addrinfo

def pf_ef(idx, ts, (prio, edp, dp, pf, ps, sa), data, j1939):
    if dp:
        return 'Proprietary A2'

    return 'Proprietary A1'
    
def pf_ff(idx, ts, (prio, edp, dp, pf, ps, sa), data, j1939):
    pgn = "%.2x :: %.2x:%.2x - %s" % (sa, pf,ps, data.encode('hex'))
    return "Proprietary B %s" % pgn

pgn_pfs = {
        0x93:   ("Name Management", None),
        0xc9:   ("Request2",        pf_c9),
        0xca:   ('Transfer',        None),
        0xe8:   ("ACK        ",     None),
        0xea:   ("Request      ",   pf_ea),
        0xeb:   ("TP.DT (WTF?)",    None),
        0xec:   ("TP.CM (WTF?)",    None),
        0xee:   ("Address Claim",   pf_ee),
        0xef:   ("Proprietary",     pf_ef),
        #0xfe:   ("Command Address", None),
        0xff:   ("Proprietary B",   pf_ff),
        }


class J1939Interface(cancat.CanInterface):
    _msg_source_idx = J1939MSGS
    def __init__(self, port=None, baud=cancat.baud, verbose=False, cmdhandlers=None, comment='', load_filename=None, orig_iface=None, process_can_msgs=True, promisc=True):
        self._last_recv_idx = -1
        self._threads = []
        self._j1939_filters = []
        self._j1939_msg_events = {}
        self._j1939queuelock = threading.Lock()
        self._TPmsgParts = {}
        self.maxMsgsPerPGN = 0x200
        self._j1939_msg_listeners = []
        self.promisc = promisc

        cancat.CanInterface.__init__(self, port=port, baud=baud, verbose=verbose, cmdhandlers=cmdhandlers, comment=comment, load_filename=load_filename, orig_iface=orig_iface)

        # setup the message handler event offload thread
        if self._config.get('myIDs') is None:
            self._config['myIDs'] = []

        self._mhe_queue = Queue.Queue()
        mhethread = threading.Thread(target=self._mhe_runner)
        mhethread.setDaemon(True)
        mhethread.start()
        self._threads.append(mhethread)

        self.register_handler(CMD_CAN_RECV, self._j1939_can_handler)

        if process_can_msgs:
            self.processCanMessages()

        # restore other config items

    def processCanMessages(self, delete=True):
        for msg in self.recvall(CMD_CAN_RECV):
            self._j1939_can_handler(msg, None)

    def setPromiscuous(self, promisc=True):
        '''
        Determines whether messages not destined for an ID I currently own (self._config['myIDs'] are kept/handled or discarded
        '''
        self.promisc = promisc

    def addID(self, newid):
        if newid not in self._config['myIDs']:
            self._config['myIDs'].append(newid)

    def delID(self, curid):
        if curid in self._config['myIDs']:
            self._config['myIDs'].remove(curid)

    def J1939xmit(self, pf, ps, sa, data, prio=6, edp=0, dp=0):
        if len(data) < 8:
            arbid = emitArbid(prio, edp, dp, pf, ps, sa)
            # print "TX: %x : %r" % (arbid, data.encode('hex'))
            self.CANxmit(arbid, data, extflag=1)
            return

        self._j1939xmit_tp(pf, ps, sa, data, prio, edp, dp)

    def _j1939xmit_tp(self, pf, ps, sa, message, prio=6, edp=0, dp=0):
        pgn2 = (edp << 1) | dp
        pgn1 = pf
        if pgn1 < 240:
            pgn0 = 0
        else:
            pgn0 = ps

        msgs = ['%c'%(x+1) + message[x*7:(x*7)+7] for x in range((len(message)+6)/7)]
        if len(msgs) > 255:
            raise Exception("J1939xmit_tp: attempt to send message that's too large")

        cm_msg = struct.pack('<BHBBBBB', CM_RTS, len(message), len(msgs), 0xff, 
                pgn2, pgn1, pgn0)

        arbid = emitArbid(prio, edp, dp, PF_TP_CM, ps, sa)
        # print "TXe: %x : %r" % (arbid, cm_msg.encode('hex'))
        self.CANxmit(arbid, cm_msg, extflag=1)
        time.sleep(.01)  # hack: should watch for CM_CTS
        for msg in msgs:
            #self.J1939xmit(PF_TP_DT, ps, sa, msg, prio=prio)
            arbid = emitArbid(prio, edp, dp, PF_TP_DT, ps, sa)
            print "TXe: %x : %r" % (arbid, msg.encode('hex'))
            self.CANxmit(arbid, msg, extflag=1)

        # hack: should watch for CM_EOM

    def _reprCanMsg(self, idx, ts, arbtup, data, comment=None):
        #print "_reprCanMsg: %r   %r" % (args, kwargs)

        if comment is None:
            comment = ''

        prio, edp, dp, pf, ps, sa = arbtup

        # give name priority to the Handler, then the manual name (this module), then J1939PGNdb
        pfmeaning, handler = pgn_pfs.get(pf, ('',None))
        nextline = ''

        if handler is not None:
            enhanced = handler(idx, ts, arbtup, data, self)
            if enhanced == cancat.DONT_PRINT_THIS_MESSAGE:
                return enhanced

            if enhanced is not None:
                if type(enhanced) in (list, tuple) and len(enhanced):
                    pfmeaning = enhanced[0]
                    if len(enhanced) > 1:
                        nextline = '\n'.join(list(enhanced[1:]))

                    # if we get multiple lines and the first is DONT_PRINT_THIS_MESSAGE, 
                    # then just return nextline
                    if pfmeaning == cancat.DONT_PRINT_THIS_MESSAGE:
                        return nextline

                    nextline = '\n' + nextline

                else:
                    pfmeaning = enhanced

        elif not len(pfmeaning):
            pgndata = parsePGNData(pf, ps, data)
            pfmeaning = pgndata.get('pgndata').get('Name')
            lines = []

            if self.verbose:
                for spnum, spdict, spunit, spdata, sprepr in pgndata.get('spns'):
                    spnName = spdict.get('Name')
                    lines.append('      SPN(%d): %-20s \t %s' % (spnum, sprepr, spnName))

                if len(lines):
                    nextline = '\n' + '\n'.join(lines)

        return "%.8d %8.3f pri/edp/dp: %d/%d/%d, PG: %.2x %.2x  Source: %.2x  Data: %-18s  %s\t\t%s%s" % \
                (idx, ts, prio, edp, dp, pf, ps, sa, data.encode('hex'), pfmeaning, comment, nextline)

    def _j1939_can_handler(self, tsmsg, none):
        '''
        this function is run for *Every* received CAN message... and is executed from the 
        XMIT/RECV thread.  it *must* be fast!
        '''
        #print repr(self), repr(cmd), repr(tsmsg)
        ts, message = tsmsg
        arbid, data = self._splitCanMsg(message)
        arbtup = parseArbid(arbid)
        prio, edp, dp, pf, ps, sa = arbtup

        # if i don't care about this message... bail. (0xef+ is multicast)
        if pf < 0xef and ps not in self._config['myIDs'] and not self.promisc:
            return

        if pf == 0xeb:
            self.queueMessageHandlerEvent(self.eb_handler, arbtup, data, ts)
        elif pf == 0xec:
            self.queueMessageHandlerEvent(self.ec_handler, arbtup, data, ts)
        else:
            self.queueMessageHandlerEvent(self._submitJ1939Message, arbtup, data, ts)

        #print "submitted message: %r" % (message.encode('hex'))


    def queueMessageHandlerEvent(self, pfhandler, arbtup, data, ts):
        ''' 
        this is run in the XMIT/RECV thread and is intended to handle offloading the data fast
        '''
        self._mhe_queue.put((pfhandler, arbtup, data, ts))

    def _mhe_runner(self):
        ''' 
        runs the mhe thread, which is offloaded so that the message-handling thread can keep going
        '''
        while not self._config.get('shutdown'):
            while self._config['go']:
                worktup = None
                try:
                    worktup = self._mhe_queue.get(1)
                    if worktup is None:
                        continue

                    pfhandler, arbtup, data, ts = worktup
                    pfhandler(arbtup, data, ts)

                except Exception, e:
                    print "MsgHandler ERROR: %r (%r)" % (e, worktup)
                    if self.verbose:
                        sys.excepthook(*sys.exc_info())

            time.sleep(1)

    def _submitJ1939Message(self, arbtup, message, timestamp=None):
        '''
        submits a message to the cmd mailbox.  creates mbox if doesn't exist.
        *threadsafe*
        often runs in the MHE thread
        '''
        print "_submitJ1939Message"
        if timestamp is None:
            timestamp = time.time()

        prio, edp, dp, pf, ps, sa = arbtup
        pgn = (pf<<8) | ps
        datarange = (edp<<1) | dp

        if len(self._j1939_filters):
            lcls = locals()
            for advf in self._j1939_filters:
                try:
                    if not eval(advf, lcls):
                        return
                except Exception, e:
                    print "_submitJ1939Message advfilter ERROR: %r" % e
                    return

        self._j1939queuelock.acquire()
        try:
            # do we want to break things apart into PGN mboxes at this point?  if so, we have to also allow 
            # subscription at this level for things like sniffing.  Like this:
            handled = False
            for listener in self._j1939_msg_listeners:
                try:
                    contnu = listener(arbtup, message)
                    if contnu: 
                        handled = True

                except Exception, e:
                    self.log('_submitJ1939Message: ERROR: %r' % e)
            
            # check for any J1939 registered handlers (using the default system handlers):
            cmdhandler = self._cmdhandlers.get(J1939MSGS)
            if cmdhandler is not None:
                handled2 = cmdhandler(tsmsg, self)

            if handled and handled2:
                #print "handled"
                return

            ##::: TODO, make this a listener.  if at all...
            #dr = self._messages.get(datarange)
            #if dr is None:
                #dr = {}
                #self._messages[datarange] = dr
            #
            ## factor in multicast vs. unicast...
            #mbox = dr.get(pf)
            #if mbox is None:
                #mbox = []
                #dr[pf] = mbox
                #self._j1939_msg_events[pf] = threading.Event()

            # file in the mailbox
            mbox = self._messages.get(J1939MSGS)
            if mbox is None:
                mbox = []
                self._messages[J1939MSGS] = mbox

            msgevt = self._j1939_msg_events.get(J1939MSGS)
            if msgevt is None:
                msgevt = threading.Event()
                self._j1939_msg_events[J1939MSGS] = msgevt

            #mbox.append((pf, ps, sa, edp, dp, prio, timestamp, message))
            mbox.append((timestamp, arbtup, message))
            msgevt.set()
            ##self._j1939_msg_events[pf].set()
            # note: this event will trigger for any of the data ranges, as long as the PF is correct... this may be a problem.
            # FIXME: come back to this...

        except Exception, e:
            self.log("_submitMessage: ERROR: %r" % e, -1)
            if self.verbose:
                sys.excepthook(*sys.exc_info())

        finally:
            self._j1939queuelock.release()

    def getJ1939MsgCount(self):
        j1939Msgs = self._messages.get(J1939MSGS)
        if j1939Msgs is None:
            return 0
        return len(j1939Msgs)

    def subscribe(self, msg_handler):
        if msg_handler not in self._j1939_msg_listeners:
            self._j1939_msg_listeners.append(msg_handler)

    def unsubscribe(self, msg_handler):
        if msg_handler in self._j1939_msg_listeners:
            self._j1939_msg_listeners.remove(msg_handler)

    def ec_handler(j1939, arbtup, data, ts):
        '''
        special handler for TP_CM messages

        pgn2 is PS/DA
        pgn1 is PF
        pgn0 is prio/edp/dp
        '''
        def tp_cm_10(arbtup, data, j1939):     # RTS
            (prio, edp, dp, pf, da, sa) = arbtup

            (cb, totsize, pktct, maxct,
                    pgn2, pgn1, pgn0) = struct.unpack('<BHBBBBB', data)
            
            # check for old stuff
            extmsgs = j1939.getTPmsgParts(da, sa)
            if extmsgs is not None and len(extmsgs['msgs']):
                pgn2 = extmsgs['pgn2']
                pgn1 = extmsgs['pgn1']
                pgn0 = extmsgs['pgn0']
                j1939.saveTPmsg(da, sa, (pgn2, pgn1, pgn0), meldExtMsgs(extmsgs), TP_DIRECT_BROKEN)
                j1939.clearTPmsgParts(da, sa)

            # store extended message information for other stuff...
            extmsgs = j1939.getTPmsgParts(da, sa, create=True)
            extmsgs['ts'] = ts
            extmsgs['sa'] = sa
            extmsgs['da'] = da
            extmsgs['pgn2'] = pgn2
            extmsgs['pgn1'] = pgn1
            extmsgs['pgn0'] = pgn0
            extmsgs['maxct'] = maxct
            extmsgs['length'] = pktct
            extmsgs['totsize'] = totsize
            extmsgs['type'] = TP_DIRECT
            extmsgs['adminmsgs'].append((arbtup, data))

            # RESPOND!
            if da in j1939._config['myIDs']:
                response = struct.pack('<BBBHBBB', CM_CTS, pktct, 1, 0, pgn2, pgn1, pgn0)
                j1939.J1939xmit(0xec, sa, da, response, prio)

        def tp_cm_11(arbtup, data, j1939):     # CTS
            (prio, edp, dp, pf, da, sa) = arbtup

            (cb, maxpkts, nextpkt, reserved,
                    pgn2, pgn1, pgn0) = struct.unpack('<BBBHBBB', data)

            # store extended message information for other stuff...
            extmsgs = j1939.getTPmsgParts(da, sa)
            if extmsgs is None:
                return

            extmsgs['adminmsgs'].append((arbtup, data))

            # SOMEHOW WE TRIGGER THE CONTINUATION OF TRANSMISSION

        def tp_cm_13(arbtup, data, j1939):     # EOM
            (prio, edp, dp, pf, da, sa) = arbtup

            (cb, totsize, pktct, maxct,
                    pgn2, pgn1, pgn0) = struct.unpack('<BHBBBBB', data)

            # print out extended message and clear the buffers.
            extmsgs = j1939.getTPmsgParts(da, sa)
            if extmsgs is None:
                return 

            extmsgs['adminmsgs'].append((arbtup, data))

            j1939.clearTPmsgParts(da, sa)
            # Coolio, they just confirmed receipt, we're done!
            # Probably need to trigger some mechanism telling the originator

        def tp_cm_20(arbtup, data, j1939):     # BROADCAST MESSAGE (BAM)
            (prio, edp, dp, pf, da, sa) = arbtup

            (cb, totsize, pktct, reserved,
                    pgn2, pgn1, pgn0) = struct.unpack('<BHBBBBB', data)

            # check for old stuff
            extmsgs = j1939.getTPmsgParts(da, sa)
            if extmsgs is not None and len(extmsgs['msgs']):
                pgn2 = extmsgs['pgn2']
                pgn1 = extmsgs['pgn1']
                pgn0 = extmsgs['pgn0']
                j1939.saveTPmsg(da, sa, (pgn2, pgn1, pgn0), meldExtMsgs(extmsgs), TP_DIRECT_BROKEN)

                j1939.clearTPmsgParts(da, sa)

            # store extended message information for other stuff...
            extmsgs = j1939.getTPmsgParts(da, sa, create=True)
            extmsgs['ts'] = ts
            extmsgs['sa'] = sa
            extmsgs['da'] = da
            extmsgs['pgn2'] = pgn2
            extmsgs['pgn1'] = pgn1
            extmsgs['pgn0'] = pgn0
            extmsgs['maxct'] = 0
            extmsgs['length'] = pktct
            extmsgs['totsize'] = totsize
            extmsgs['type'] = TP_BAM
            extmsgs['adminmsgs'].append((arbtup, data))

        # call the right TP_CM handler
        tp_cm_handlers = {
                CM_RTS:     ('RTS',           tp_cm_10),
                CM_CTS:     ('CTS',           tp_cm_11),
                CM_EOM:     ('EndOfMsgACK',   tp_cm_13),
                CM_BAM:     ('BAM-Broadcast', tp_cm_20),
                CM_ABORT:   ('Abort',         None),
                }

        cb = ord(data[0])
        #print "ec: %.2x%.2x %.2x" % (arbtup[3], arbtup[4], cb)

        htup = tp_cm_handlers.get(cb)
        if htup is not None:
            subname, cb_handler = htup

            if cb_handler is not None:
                cb_handler(arbtup, data, j1939)

    def eb_handler(j1939, arbtup, data, ts):
        '''
        special handler for TP_DT messages
        '''
        (prio, edp, dp, pf, da, sa) = arbtup
        if len(data) < 1:
            j1939.log('pf=0xeb: TP ERROR: NO DATA!')
            return

        extmsgs = j1939.getTPmsgParts(da, sa)
        if extmsgs is None:
            j1939.log("TP_DT: haven't received TP_CM control setup, skipping")
            return

        extmsgs['msgs'].append((arbtup, data))
        if len(extmsgs['msgs']) >= extmsgs['length']:
            # we're done building this message, submit it!
            #print "eb_handler: saving: %r %r" % (len(extmsgs['msgs']) , extmsgs['length'])
            pgn2 = extmsgs['pgn2']
            pgn1 = extmsgs['pgn1']
            pgn0 = extmsgs['pgn0']
            mtype = extmsgs['type']

            j1939.saveTPmsg(da, sa, (pgn2, pgn1, pgn0), meldExtMsgs(extmsgs), mtype)
            j1939.clearTPmsgParts(da, sa)

            # if this is the end of a message to *me*, reply accordingly
            if da in j1939._config['myIDs']:
                if mtype is None:
                    j1939.log("TP_DT_handler: missed beginning of message, not sending EOM: %r" % \
                            repr(extmsgs), 1)
                    return

                j1939.log("tp_stack: sending EOM  extmsgs: %r" % extmsgs, 1)
                pgn2 = extmsgs['pgn2']
                pgn1 = extmsgs['pgn1']
                pgn0 = extmsgs['pgn0']
                totsize = extmsgs['totsize']
                maxct = extmsgs['maxct']
                pktct = extmsgs['length']

                data = struct.pack('<BHBBBBB', CM_EOM, totsize, pktct, maxct, pgn2, pgn1, pgn0)
                j1939.J1939xmit(PF_TP_CM, sa, da,  data, prio=prio)

    # functions to support the J1939TP Stack (real stuff, not just repr)
    '''
    these functions support TP messaging.  Message parts are stored as PF lists within DA dicts within SA dicts.
    ie.:
        self_TPmsgParts[sa][da][pf]

    this allows for clearing of entire parts of the transient stack easily by SA.
    The main message stack has a *different* hierarchy based on what's easiest for developing client code to access.
    '''
    def getTPmsgParts(self, da, sa, create=False):
        '''
        # functions to support the J1939TP Stack (real stuff, not just repr)
        returns a message list for a given source and destination (sa, da)

        if no list exists for this pairing, one is created and an empty list is returned
        '''
        msglists = self._TPmsgParts.get(sa)
        if msglists is None:
            msglists = {}
            self._TPmsgParts[sa] = msglists

        mlist = msglists.get(da)
        if mlist is None and create:
            # create something new
            mlist = {'length':0, 
                    'msgs':[], 
                    'type':None, 
                    'adminmsgs':[], 
                    'pgn0':None, 
                    'pgn1':None, 
                    'pgn2':None,   
                    'totsize':0,
                    'maxct':0xff,
                    'sa' : sa,
                    'da' : da,
                    }
            msglists[da] = mlist

        return mlist

    def clearTPmsgParts(self, da, sa):
        '''
        # functions to support the J1939TP Stack (real stuff, not just repr)
        clear out extended messages metadata.

        if da is None, this clears *all* message data for a given source address

        returns whether the thing deleted exists previously
        * if da is None, returns whether the sa had anything previously
        * otherwise, if the list 
        '''
        exists = False
        if da is None:
            msglists = self._TPmsgParts.get(sa)
            exists = bool(msglists is not None and len(msglists))
            self._TPmsgParts[sa] = {}
            return exists

        msglists = self._TPmsgParts.get(sa)
        if msglists is None:
            msglists = {}
            self._TPmsgParts[sa] = msglists

        if da in msglists:
            msglists.pop(da)
            return True

        return False

    def saveTPmsg(self, da, sa, pgn, msg, tptype):
        '''
        # functions to support the J1939TP Stack (real stuff, not just repr)
        store a TP message.
        '''
        pgn2, pgn1, pgn0 = pgn

        ps = pgn2
        pf = pgn1

        prio = pgn0 >> 2
        edp = (pgn0 >> 1) & 1
        dp = pgn0 & 1

        if da != ps:
            print "saveTPmsg: WARNING: da: 0x%x  but ps: 0x%x.  using ps" % (da, ps)
            print da, sa, pgn, repr(msg)
        arbtup = prio, edp, dp, pf, ps, sa
        self._submitJ1939Message(arbtup, msg)

    def _getLocals(self, idx, ts, arbtup, data):
        #print "getLocals:",idx, ts, arbtup, data
        prio, edp, dp, pf, ps, sa = arbtup
        pgn = (pf << 8) | ps
        lcls = {'idx': idx,
                'ts': ts,
                'data': data,
                'priority': prio,
                'edp': edp,
                'dp': dp,
                'pf': pf,
                'ps': ps,
                'sa': sa,
                'pgn': pgn,
                'da': ps,
                'ge': ps,
                }

        return lcls

    def genCanMsgs(self, start=0, stop=None, arbids=None, tail=False, maxsecs=None):
        '''
        CAN message generator.  takes in start/stop indexes as well as a list
        of desired arbids (list)

        maxsecs limits the number of seconds this generator will go for.  it's intended
        for use with tail
        '''

        messages = self.getCanMsgQueue()
        if messages is None and not tail:
            return

        # get the ts of the first received message
        if messages is not None and len(messages):
            startts = messages[0][0]
        else:
            startts = time.time()

        if start is None:
            start = self.getJ1939MsgCount()

        if stop is None or tail:
            stop = len(messages)
        else:
            stop = stop + 1 # This makes the stop index inclusive if specified

        starttime = time.time()

        idx = start
        while tail or idx < stop:
            # obey our time restrictions
            # placed here to ensure checking whether we're receiving messages or not
            if maxsecs is not None and time.time() > maxsecs+starttime:
                return
        
            # If we start sniffing before we receive any messages, 
            # messages will be "None". In this case, each time through
            # this loop, check to see if we have messages, and if so,
            # re-create the messages handle
            if messages is None:
                messages = self.getCanMsgQueue()
        
            # if we're off the end of the original request, and "tailing"
            if tail and idx >= stop:
                msgqlen = len(messages) 
                self.log("stop=%d  len=%d" % (stop, msgqlen), 3)

                if stop == msgqlen:
                    self.log("waiting for messages", 3)
                    # wait for trigger event so we're not constantly polling
                    self._msg_events[self._msg_source_idx].wait(1)
                    self._msg_events[self._msg_source_idx].clear()
                    self.log("received 'new messages' event trigger", 3)

                # we've gained some messages since last check...
                stop = len(messages)
                continue    # to the big message loop.

            # now actually handle messages
            # here's where we get J1939 specific...
            #print messages[idx]
            ts, arbtup, data = messages[idx]
            #datatup = self._splitCanMsg(msg)

            # make ts an offset instead of the real time.
            ts -= startts

            #if arbids is not None and arbid not in arbids:
            #    # allow filtering of arbids
            #    idx += 1
            #    continue

            yield((idx, ts, arbtup, data))
            idx += 1


    def J1939recv(self, pf, ps, sa, msgcount=1, timeout=1, start_msg=None, update_last_recv=True):
        out = []

        if start_msg is None:
            start_msg = self._last_recv_idx

        #for msg in self.filterCanMsgs(start_msg=start_msg, advfilters=advfilters, tail=True, maxsecs=timeout):
            #(idx, ts, arbid, data) = msg
            #out.append(msg)
            #self._last_recv_idx = msg[0]
        # FIXME: add in the wait/set interaction for lower perf impact.
        startts = time.time()
        cur = start_msg
        mque = self._messages[J1939MSGS]
        while time.time() < (startts + timeout):
            if cur >= len(mque):
                time.sleep(.001)
                continue

            ts, arbtup, msg = mque[cur]
            cur += 1
           
            # we have a message now, does the PGN match?
            mprio, medp, mdp, mpf, mps, msa = arbtup
            if mpf != pf or mps != ps or msa != sa:
                continue

            # it's passed the checks... add it to the queue
            out.append((ts, arbtup, msg))

            if len(out) >= msgcount:
                break


        # if we actually found something, and we wanted to update last recvd...
        if len(out) and update_last_recv:
            self._last_recv_idx = cur

        return out
    def J1939recv_loose(self, pf=(), ps=None, sa=None, msgcount=1, timeout=1, start_msg=None, update_last_recv=True):
        out = []

        if start_msg is None:
            start_msg = self._last_recv_idx

        #for msg in self.filterCanMsgs(start_msg=start_msg, advfilters=advfilters, tail=True, maxsecs=timeout):
            #(idx, ts, arbid, data) = msg
            #out.append(msg)
            #self._last_recv_idx = msg[0]
        # FIXME: add in the wait/set interaction for lower perf impact.
        startts = time.time()
        cur = start_msg
        mque = self._messages[J1939MSGS]
        while time.time() < (startts + timeout):
            if cur >= len(mque):
                time.sleep(.001)
                continue

            ts, arbtup, msg = mque[cur]
            cur += 1
           
            # we have a message now, does the PGN match? (loose matching)
            mprio, medp, mdp, mpf, mps, msa = arbtup
            if pf is not None:
                if type(pf) in (tuple, list):
                    if mpf not in pf:
                        continue
                else:
                    if mpf != pf:
                        continue

            if ps is not None:
                if type(ps) in (tuple, list):
                    if mps not in ps:
                        continue
                else:
                    if mps != ps:
                        continue

            if sa is not None:
                if type(sa) in (tuple, list):
                    if msa not in sa:
                        continue
                else:
                    if msa != sa:
                        continue

            # it's passed the checks... add it to the queue
            out.append((ts, arbtup, msg))

            if len(out) >= msgcount:
                break


        # if we actually found something, and we wanted to update last recvd...
        if len(out) and update_last_recv:
            self._last_recv_idx = cur

        return out

    def J1939xmit_recv(self, pf, ps, sa, data, recv_count=1, prio=6, edp=0, dp=0, timeout=1, expected_pf=None):
        msgidx = self.getCanMsgCount()
        # FIXME: filter on the expected response PGN
        if expected_pf is None:
            expected_pf = pf

        res = self.J1939xmit(pf, ps, sa, data, prio, edp, dp)
        res = self.J1939recv(expected_pf, sa, ps, recv_count, timeout, start_msg=msgidx)

        return res


    def J1939_Request(self, rpf, rda_ge=0, redp=0, rdp=0, da=0xff, sa=0xfe, prio=0x6, recv_count=255, timeout=2, expected_pf=None):
        pgnbytes = [rda_ge, rpf, redp<<1 | rdp]
        data = ''.join([chr(x) for x in pgnbytes])
        data += '\xff' * (8-len(data))

        if expected_pf is None:
            expected_pf = rpf

        self.J1939xmit(PF_RQST, da, sa, data)
        msgs = self.J1939recv_loose(pf=expected_pf, msgcount=10, timeout=timeout)
        return msgs

    def J1939_ClaimAddress(self, addr, name=0x4040404040404040, prio=6):
        data = struct.pack(">Q", name)
        out = self.J1939xmit_recv(pf=PF_ADDRCLAIM, ps=0xff, sa=addr, data=data, recv_count=10, prio=prio<<2, timeout=2, expected_pf=0xee)
        self.addID(addr)
        return out

    def J1939_ArpAddresses(self):
        '''
        Sends a request for all used addresses... not fully tested
        '''
        #idx = self.getCanMsgCount()
        msgs = self.J1939_Request(PF_ADDRCLAIM, recv_count=255, timeout=3)

        '''
        # FIXME: these are way too loose, for discovery only. tighten down.
        recv_filters = [
                'pf < 0xf0',
                #'pf == 0xee',
                ]

        msgs = self.J1939recv(msgcount=200, timeout=3, advfilters=recv_filters, start_msg=idx)
        '''
        for msg in msgs:
            try:
                msgrepr = self._reprCanMsg(*msg)
                if msgrepr != cancat.DONT_PRINT_THIS_MESSAGE:
                    print msgrepr
            except Exception, e:
                print e
        '''
        example (from start of ECU):
        00000000 1545142410.990 pri/edp/dp: 6/0/0, PG: ea ff  Source: fe  Len: 03, Data: 00ee00              Request
        00000001 1545142411.077 pri/edp/dp: 6/0/0, PG: ee ff  Source: 00  Len: 08, Data: 4cca4d0100000000    Address Claim: id: 0xdca4c mfg: Cummins Inc (formerly Cummins Engine Co) Columbus, IN USA
    
        currently ours:
        00001903 1545142785.127 pri/edp/dp: 6/0/0, PG: ea ff  Source: fe  Len: 03, Data: 00ee00              Request

        '''

MAX_WORD = 64
bu_masks = [(2 ** (i)) - 1 for i in range(8*MAX_WORD+1)]

def parsePGNData(pf, ps, msg):

    # piece the correct PGN together from PF/PS
    if pf < 0xec:
        pgn = pf << 8
    else:
        pgn = (pf << 8) | ps

    # grab the PGN data
    res = J1939PGNdb.get(pgn)
    out = {'pgn': pgn, 'pgndata': res}

    spnlist = res.get('SPNs')
    spndata = []

    var_len_idx = 0
    for spnum in spnlist:
        # get SPN data
        spn = J1939SPNdb.get(spnum)
        if spn is None:
            continue

        # graciously refactored code from TruckDevil (hey LBD!)
        spnlen = spn.get('SPNLength')
        pgnlen = spn.get('PGNLength')
        spnName = spn.get('Name')
        spnRepr = ''
        datanum = -1
        units = spn.get("Units")

        # skip variable-length PGNs for now
        if (type(pgnlen) == str and 'ariable' in pgnlen):
            datablob = msg

        else:
            startBit = spn.get('StartBit')
            endBit = spn.get('EndBit')

            startByte = startBit // 8
            startBitO = startBit % 8
            endByte = (endBit + 7) // 8
            endBitO = endBit % 8

            datablob = msg[startByte:endByte]

        #print "sb: %d\t eb: %d\t sB:%d\t SBO:%d\t eB:%d\t eBO:%d\t %r" % (startBit, endBit, startByte, startBitO, endByte, endBitO, datablob)

        if units == 'ASCII':
            spnRepr = repr(datablob)
            datanum = datablob

        else:
            try:
                # carve out the number
                datanum = 0
                numbytes = struct.unpack('%dB' % len(datablob), datablob)
                for i, n in enumerate(numbytes):
                    datanum |= (n << (8*i))
                    #print "datanum (working): 0x%x   ::  0x%x" % (n, datanum)
                    #datanum <<= 8

                datanum >>= (7 - endBitO)
                #print "datanum: %x" % datanum

                mask = bu_masks[endBit - startBit + 1]
                datanum &= mask

                offset = spn.get('Offset')
                if offset is not None:
                    datanum += offset
                #print "datanum: %x (mask: %x)" % (datanum, mask)

                # make sense of the number based on units
                if units == 'bit':
                    meaning = ''
                    bitdecode = J1939BitDecodings.get(spnum)
                    if bitdecode is not None:
                        meaning = bitdecode.get(datanum)

                    spnRepr = '0x%x (%s)' % (datanum, meaning)

                elif units == 'binary':
                    spnRepr = "%s (%x)" % (bin(int(datanum)), datanum)

                elif units == '%':
                    spnRepr = "%d%%" % datanum

                else:
                    # some other unit with a resolution
                    resolution = spn.get('Resolution')
                    if resolution is not None:
                        datanum *= resolution

                    spnRepr = '%.3f %s' % (datanum, units)

            except Exception as e:
                spnRepr = "ERROR"
                print "SPN: %r %r (%r)" % (e, msg, spn)
                traceback.print_exc()

        spndata.append((spnum, spn, units, datanum, spnRepr))

    out['spns'] = spndata
    return out




# ECU class

import time

from cancat.uds import utils, UDS, SVC_SECURITY_ACCESS, NegativeResponseException 
from cancat.utils import log


class ScanClass(UDS):
    def __init__(self, c, tx_arbid, rx_arbid=None, verbose=True, extflag=0, timeout=3.0):
        super(ScanClass, self).__init__(c, tx_arbid, rx_arbid, verbose=verbose, extflag=extflag, timeout=timeout)

    def get_key(self, session=None, auth_lvl=None):
        # Placeholder function that can be replaced in a subclass to return 
        # a valid key that will be attempted during the auth scan.
        return ''

    def _key_from_seed(self, seed, secret):
        # As long as Scanning
        self.seed = {
            'secret': secret,
            'seed': seed,
            'key': '',
        }

        # For our dumb seed/key scanning purposes, just "send" the secret to the 
        # ECU.
        return secret

    def _do_Function(self, func, data=None, subfunc=None, service=None):
        # Special case to capture the final "key" being sent
        if func == SVC_SECURITY_ACCESS and data is not None:
            self.seed['key'] = data
        return super(ScanClass, self)._do_Function(func, data=data, subfunc=subfunc, service=service)


class ECU(object):
    # Add the kwargs param so we can construct an ECUAddress out of a dictionary 
    # that has extra stuff in it
    def __init__(self, c, addr, scancls=None, timeout=3.0, delay=None, sessions=None, **kwargs):
        self._addr = addr # (arb_id, resp_id, extflag)
        if scancls is None:
            self._scancls = ScanClass
        else:
            self._scancls = scancls
        self._timeout = timeout
        self._delay = delay
        self.c = c

        if sessions is not None:
            # TODO: probably need to validate/massage this object instead of 
            #       assuming it'll be correctly formatted?
            self._sessions = sessions
        else:
            self._sessions = {1: {'dids': {}}}

    def export(self):
        import collections
        data = collections.OrderedDict()
        data['tx_arbid'] = self._addr.tx_arbid
        data['rx_arbid'] = self._addr.rx_arbid
        data['extflag'] = self._addr.extflag
        data['sessions'] = self._sessions
        return data

    def did_read_scan(self, did_range, rescan=False):
        # Only do a scan if we don't already have data, unless rescan is set
        if not self._sessions[1]['dids'] or rescan:
            arb, resp, ext = self._addr
            log.msg('{} starting DID scan'.format(self._addr))
            # The DID scan is more reliable using the standard UDS timeout 
            # because of the length of time that block transfers can take
            u = self._scancls(self.c, arb, resp, extflag=ext, verbose=False, timeout=3.0)
            self._sessions[1]['dids'].update(utils.did_read_scan(u, did_range, delay=self._delay))

    def did_write_scan(self, did_range, rescan=False):
        # Only do a scan if we don't already have data, unless rescan is set
        if not self._sessions[1]['write_dids'] or rescan:
            # Attempt to write an empty array, which probably won't succeed?  
            # but if it does we're pretty screwed.
            arb, resp, ext = self._addr
            log.msg('{} starting DID write scan'.format(self._addr))
            u = self._scancls(self.c, arb, resp, extflag=ext, verbose=False, timeout=self._timeout)
            self._write_dids.update(utils.did_write_scan(u, did_range, b'', delay=self._delay))

    def session_scan(self, session_range, rescan=False, rescan_did_range=None, recursive_scan=True):
        arb, resp, ext = self._addr
        # Unfortunately session scanning (and the later DID scanning) is more 
        # reliable with the standard 3 second timeout
        u = self._scancls(self.c, arb, resp, extflag=ext, verbose=False, timeout=3.0)

        log.msg('{} starting session scan'.format(self._addr))

        # Only scan for new sessions if the session list consists only of session 1
        if len(self._sessions) == 1:
            new_sessions = utils.session_scan(u, session_range, delay=self._delay,
                    recursive_scan=recursive_scan)
            self._sessions.update(new_sessions)

        # For each session that was found, go through the list of DIDs and 
        # identify which DIDs can be read in this session
        for sess in self._sessions:
            if sess != 1 and 'resp' in self._sessions[sess] and (rescan or \
                    'dids' not in self._sessions[sess] or len(self._sessions[sess]['dids']) == 0):
                try:
                    with utils.new_session(u, sess, self._sessions[sess]['prereqs'], True):
                        log.debug('{} session {} ({}) re-reading DIDs'.format(
                            self._addr, sess, self._sessions[sess]['prereqs']))

                        self._sessions[sess]['dids'] = {}
                        # If rescan is set do a full DID scan instead of the short 
                        # scan of only existing DIDs
                        if rescan:
                            results = utils.did_read_scan(u, rescan_did_range, delay=self._delay)
                            self._sessions[sess]['dids'].update(results)
                        else:
                            valid_did_range = [d for d in self._sessions[1]['dids']]
                            results = utils.did_read_scan(u, valid_did_range, delay=self._delay)
                            self._sessions[sess]['dids'].update(results)
                except NegativeResponseException as e:
                    log.error('Failed to enter session {} ({}) to re-scan DIDs, try again later: {}'.format(
                        sess, self._sessions[sess]['prereqs'], e))

    def auth_scan(self, auth_range, rescan=False):
        arb, resp, ext = self._addr
        u = self._scancls(self.c, arb, resp, extflag=ext, verbose=False, timeout=self._timeout)
        u.StartTesterPresent(request_response=False)

        for sess in self._sessions:
            if sess != 1 and \
                    ('auth' not in self._sessions[sess] or \
                    len(self._sessions[sess]['auth']) == 0 or rescan):
                log.msg('{} session {} starting auth scan'.format(self._addr, sess))
                with utils.new_session(u, sess, self._sessions[sess]['prereqs'], True):
                    # Pass the get_key() function in the UDS scan class through
                    results = utils.auth_scan(u, auth_range,
                            lambda x: u.get_key(sess, x), delay=self._delay)

                    if 'auth' in self._sessions[sess]:
                        self._sessions[sess]['auth'].update(results)
                    else:
                        self._sessions[sess]['auth'] = results

    def _try_key(self, u, auth_level, key):
        resp = utils.try_auth(u, auth_level, key)
        if resp is not None and 'err' in resp:
            # Attempt to handle a few common errors
            if 'err' in resp and resp['err'] == 0x36:
                # 0x36:'ExceedNumberOfAttempts'
                log.detail('Retrying session {}, auth {}, length {}'.format(sess, lvl, keylen))
                resp = utils.try_auth(u, level, key)
                log.detail('resp: {}'.format(resp))

            elif 'err' in resp and resp['err'] == 0x37:
                # 0x37:'RequiredTimeDelayNotExpired'
                time.sleep(1.0)
                log.detail('Retrying session {}, auth {}, length {}'.format(sess, lvl, keylen))
                resp = utils.try_auth(u, level, key)
                log.detail('resp: {}'.format(resp))
        return resp

    def _found_key_len(self, sess, auth_level):
        # error 0x35 means that the key is invalid, which also means that the 
        # length is correct
        # 0x35:'InvalidKey'
        if 'resp' in self._sessions[sess]['auth'][auth_level] or \
                ('err' in self._sessions[sess]['auth'][auth_level] and \
                 self._sessions[sess]['auth'][auth_level]['err'] == 0x35):
            return True
        else:
            return False

    def key_length_scan(self, len_range, rescan=False):
        log.msg('{} starting key/seed length scan {}'.format(self._addr, len_range))
        self.c.placeCanBookmark('canmap key_length_scan({}, delay={})'.format(len_range, self._delay))

        arb, resp, ext = self._addr
        u = self._scancls(self.c, arb, resp, extflag=ext, verbose=False, timeout=self._timeout)
        u.StartTesterPresent(request_response=False)

        # I can't think of a good way to turn this into a more generic utility 
        # function
        for sess in self._sessions:
            # Skip session 1
            if sess == 1:
                continue

            with utils.new_session(u, sess, True):
                for lvl in self._sessions[sess]['auth']:
                    if not self._found_key_len(sess, lvl) or rescan:
                        # TODO: For now, we delete the old scan data, not sure how 
                        # best to track to new vs. old key key scans otherwise
                        self._sessions[sess]['auth'][lvl] = { 'seeds': [] }
                        log.msg('{} session {} auth {} starting key length scan'.format(self._addr, sess, lvl))
                        for keylen in len_range:
                            key = '\x00' * keylen
                            log.detail('Trying session {}, auth {}, key \'{}\''.format(sess, lvl, key))
                            self.c.placeCanBookmark('SecurityAccess({}, {})'.format(i, repr(key)))
                            resp = self._try_key(u, lvl, key)
                            self._sessions[sess]['auth'][lvl].update(resp)

                            self._sessions[sess]['auth'][lvl]['seeds'].append(dict(u.seed))

                            if 'resp' in resp:
                                # Get the key attempted from the recorded seed data 
                                # in case the scanning class modified it
                                log.msg('Session {}, auth {} key found! secret {} (seed {})'.format(
                                    sess, lvl, repr(u.seed['secret']), repr(u.seed['seed'])))
                                break
                            elif resp['err'] == 0x35:
                                log.debug('Session {}, auth {} length found! {} (seed {})'.format(
                                    sess, lvl, len(u.seed['secret']), repr(u.seed['seed'])))
                                break

        if self._delay:
            time.sleep(self._delay)

        # Ensure tester present is not being sent anymore
        u.StopTesterPresent()

    def memory_read_test(self):
        raise NotImplementedError()

    def data_transfer_test(self):
        raise NotImplementedError()



#!/usr/bin/env python3
from __future__ import print_function

import sys
import readline
import rlcompleter
readline.parse_and_bind("tab: complete")
import os
from termcolor import colored, cprint

from cancat import *


intro = colored("""'CanCat, the greatest thing since J2534!'

Research Mode: enjoy the raw power of CanCat

currently your environment has an object called "c" for CanCat.  this is how
you interact with the CanCat tool:
    >>> c.ping()
    >>> c.placeCanBookmark('')
    >>> c.snapshotCanMsgs()
    >>> c.printSessionStats()
    >>> c.printCanMsgs()
    >>> c.printCanSessions()
    >>> c.CANxmit('message', )
    >>> c.CANreplay()
    >>> c.saveSessionToFile('file_to_save_session_to')
    >>> help(c)

""", "green")

if __name__ == "__main__":
    import argparse

    # Make it easy to find modules in CWD
    sys.path.append('.')
    sys.path.append("./cancat")

    interfaces = [iface[:-9] for iface in globals().keys() if iface.endswith('Interface')]
    interface_names = ', '.join(interfaces)

    bauds = [baud[4:-3] for baud in globals().keys() if baud.startswith('CAN_') and baud.endswith('BPS')]
    baud_nums = ', '.join(bauds)

    parser = argparse.ArgumentParser()
    parser.add_argument('-p', '--port', help='eg: /dev/ttyACM0')
    parser.add_argument('-f', '--filename', help='Load file (does not require CanCat device)')
    parser.add_argument('-I', '--interface', help='Use a predefined Interface (%s)' % interface_names)
    parser.add_argument('-S', '--baud', help='Set the CAN Bus Speed (%s)' % (baud_nums))

    ifo = parser.parse_args()

    interface = CanInterface

    if ifo.interface:
        ifacename = ifo.interface + "Interface"
        interface = globals().get(ifacename)
        if interface == None:
            raise Exception("Invalid interface: %s.  Must use one of the following: %s" % (ifo.interface, interface_names))

    baud_val = CAN_500KBPS
    if ifo.baud:
        baud_val = globals().get("CAN_%sBPS" % ifo.baud)
        if baud_val == None:
            raise Exception("Invalid baud: %s.  Must use one of the following: %s" % (ifo.baud, baud_nums))

    # TODO make a try/except block
    results = interactive(ifo.port, intro=intro, InterfaceClass=interface, load_filename=ifo.filename, can_baud=baud_val)
    if results == -1:
        print(colored("Error.  Try '-h' from CLI for help.", "red"))



import os
import sys
import codecs
import setuptools

version = '1.9.3.1'
packages = ['cancat','cancat/vstruct']
mods = []
pkgdata = {}
scripts = ['CanCat.py', 'J1939Cat']

dirn = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(dirn, 'README.md'), 'r') as fd:
    desc = fd.read()

setuptools.setup  (name  = 'cancat',
        version          = version,
        description      = "Multi-purpose tool for interacting with Controller Area Networks (CAN) and SAE J1939",
        long_description = desc,
        long_description_content_type='text/markdown',
        author           = 'atlas of d00m and the GRIMM CyPhy team',
        author_email     = 'atlas@r4780y.com',
        url              = 'https://github.com/atlas0fd00m/CanCat',
        download_url     = 'https://github.com/atlas0fd00m/CanCat/archive/v%s.tar.gz' % version,
        keywords         = ['can', 'controller area network', 'automotive', 'j1939', 'hacking', 'reverse engineering'],
        packages         = setuptools.find_packages(),
        package_data     = pkgdata,
        ext_modules      = mods,
        scripts          = scripts,
        install_requires = [    
                "ipython",
                "pyserial",
                "pyusb",
                "termcolor",
                "future",
            ],
        classifiers      = [
                            'Development Status :: 5 - Production/Stable',
                            'Intended Audience :: Telecommunications Industry',
                            'Topic :: Communications',
                            'License :: OSI Approved :: BSD License',
                            'Programming Language :: Python :: 3',
                            'Programming Language :: Python :: 3.6',
                            'Programming Language :: Python :: 3.7',
                            'Programming Language :: Python :: 3.8',
                            'Programming Language :: Python :: 3.9',
                           ],
        python_requires  = '>=3.6'
        )



from __future__ import print_function
from past.builtins import xrange

import unittest

import vstruct
from vstruct.primitives import *

from cStringIO import StringIO

class woot(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.lenfield = v_uint8()
        self.strfield = v_str(size=0x20)

    def pcb_lenfield(self):
        self.vsGetField('strfield').vsSetLength(self.lenfield)


class VStructTest(unittest.TestCase):

    #def setUp(self):
    #def tearDown(self):

    def test_vstruct_basicstruct(self):

        v = vstruct.VStruct()
        v.uint8 = v_uint8(1)
        v.uint16 = v_uint16(2)
        v.uint24 = v_uint24(3)
        v.uint32 = v_uint32(4)
        v.uint64 = v_uint64(5)
        v.vbytes = v_bytes(vbytes='ABCD')

        answer = '01020003000004000000050000000000000041424344'.decode('hex')
        self.assertEqual( v.vsEmit(), answer )


    def test_vstruct_basicreasign(self):
        v = vstruct.VStruct()
        v.uint8 = v_uint8(1)
        v.uint16 = v_uint16(2)
        v.uint24 = v_uint24(3)
        v.uint32 = v_uint32(4)
        v.uint64 = v_uint64(5)
        v.vbytes = v_bytes(vbytes='ABCD')

        v.uint8 = 99
        v.uint16 = 100
        v.uint24 = 101
        v.uint32 = 102
        v.uint64 = 103
        v.vbytes = '\x00\x00\x00\x00'

        answer = '63640065000066000000670000000000000000000000'.decode('hex')
        self.assertEqual( v.vsEmit(), answer )


    def test_vstruct_fieldalign(self):
        v = vstruct.VStruct()
        v._vs_field_align = True
        v.uint8 = v_uint8(0x42, bigend=True)
        v.uint16 = v_uint16(0x4243, bigend=True)
        v.uint24 = v_uint24(0x424344, bigend=True)
        v.uint32 = v_uint32(0x42434445, bigend=True)
        v.uint64 = v_uint64(0x4243444546474849, bigend=True)

        answer = '420042430000424344000000424344454243444546474849'.decode('hex')
        self.assertEqual( v.vsEmit(), answer )

    def test_vstruct_fixedpartialasign(self):
        v = vstruct.VStruct()
        v.strfield = v_str(size=30)
        v.unifield = v_wstr(size=30)

        v.strfield = 'wootwoot!'
        v.unifield = 'bazbaz'

        answer = '776f6f74776f6f7421000000000000000000000000000000000000000000620061007a00620061007a00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000'.decode('hex')

        self.assertEqual( v.vsEmit(), answer )

    def test_vstruct_lengthcallback(self):

        def updatelen(vs):
            vs.vsGetField('strfield').vsSetLength(vs.lenfield)

        v = vstruct.VStruct()
        v.lenfield = v_uint8(0x30)
        v.strfield = v_str(size=30)
        v.vsAddParseCallback('lenfield', updatelen)

        v.vsParse('\x01' + 'A' * 30)
        self.assertEqual( v.vsEmit(), '0141'.decode('hex') )


    def test_vstruct_classcallback(self):
        v = woot()
        v.vsParse('\x01' + 'A'*30)
        self.assertEqual( v.vsEmit(), '0141'.decode('hex') )

    def test_vstruct_parsefd(self):
        v = woot()
        sio = StringIO('\x01' + 'A' * 30)
        v.vsParseFd(sio)
        self.assertEqual( v.vsEmit(), '0141'.decode('hex') )

    def test_vstruct_insertfield(self):
        v = woot()
        v.vsInsertField('ifield', v_uint8(), 'strfield')
        v.vsParse('\x01BAAAAA')
        self.assertEqual( v.vsEmit(), '014241'.decode('hex') )

    def test_vstruct_floats(self):

        v = vstruct.VStruct()
        v.float4 = v_float()
        v.float8 = v_double()

        v.float4 = 99.3
        v.float8 = -400.2

        self.assertEqual( v.vsEmit(), '9a99c64233333333330379c0'.decode('hex') )

    def test_vstruct_fastparse(self):
        v = vstruct.VStruct()
        v.x = v_uint8()
        v.y = v_str(size=4)
        v.z = v_uint32()

        v.vsParse('BAAAAABCD', fast=True)

        self.assertEqual( v.x, 0x42 )
        self.assertEqual( v.y, 'AAAA' )
        self.assertEqual( v.z, 0x44434241 )

    def test_vstruct_fastparse_bigend(self):
        v = vstruct.VStruct()
        v.x = v_uint8()
        v.y = v_str(size=4)
        v.z = v_uint32(bigend=True)

        v.vsParse('BAAAAABCD', fast=True)

        self.assertEqual( v.x, 0x42 )
        self.assertEqual( v.y, 'AAAA' )
        self.assertEqual( v.z, 0x41424344 )

    def test_vstruct_varray(self):
        v = vstruct.VArray( [ v_uint8(i) for i in xrange(20) ] )
        self.assertEqual( v[2], 2 )
        v.vsParse('A' * 20)
        self.assertEqual( v[2], 0x41 )

    def test_bitfield(self):
        from vstruct.bitfield import *
        v = VBitField()
        v.vsAddField('w', v_bits(2))
        v.vsAddField('x', v_bits(3))
        v.vsAddField('y', v_bits(3))
        v.vsAddField('z', v_bits(11))
        v.vsAddField('a', v_bits(3))

        v.vsAddField('stuff', v_bits(23))
        v.vsAddField('pad', v_bits(3))
        v.vsAddField('pad2', v_bits(6))
        v.vsAddField('pad3', v_bits(2))


        v.vsParse('AAAAAAA')
        #print(v.tree())
        self.assertEqual(1, v.w)
        self.assertEqual(0, v.x)
        self.assertEqual(1, v.y)
        self.assertEqual(0x20a, v.z)
        self.assertEqual(0, v.a)
        self.assertEqual(0x282828, v.stuff)
        self.assertEqual(1, v.pad)
        self.assertEqual(16, v.pad2)
        self.assertEqual(1, v.pad3)

        self.assertEqual('AAAAAAA', v.vsEmit())

        v.vsParse('ABCDEFG')
        #print(v.tree())
        self.assertEqual(1, v.w)
        self.assertEqual(0, v.x)
        self.assertEqual(1, v.y)
        self.assertEqual(0x212, v.z)
        self.assertEqual(0, v.a)
        self.assertEqual(0x6888a8, v.stuff)
        self.assertEqual(6, v.pad)
        self.assertEqual(17, v.pad2)
        self.assertEqual(3, v.pad3)

        self.assertEqual('ABCDEFG', v.vsEmit())


        v.vsParse('zxcvbnm')
        #print(v.tree())
        self.assertEqual(1, v.w)
        self.assertEqual(7, v.x)
        self.assertEqual(2, v.y)
        self.assertEqual(0x3c3, v.z)
        self.assertEqual(0, v.a)
        self.assertEqual(0x6ecc4d, v.stuff)
        self.assertEqual(6, v.pad)
        self.assertEqual(0x1b, v.pad2)
        self.assertEqual(1, v.pad3)

        self.assertEqual('zxcvbnm', v.vsEmit())

        
        v.vsParse('asdfghj')
        #print(v.tree())
        self.assertEqual(1, v.w)
        self.assertEqual(4, v.x)
        self.assertEqual(1, v.y)
        self.assertEqual(0x39b, v.z)
        self.assertEqual(1, v.a)
        self.assertEqual(0xccced, v.stuff)
        self.assertEqual(0, v.pad)
        self.assertEqual(0x1a, v.pad2)
        self.assertEqual(2, v.pad3)

        self.assertEqual('asdfghj', v.vsEmit())

        




import time
import struct

from cancat.uds import NegativeResponseException
from cancat.uds.ecu import ScanClass

class CanInterface(object):
    """
    A fake CanInterface class for test purposes.

    All actual functions are just stubbed out, this is not a superclass of the
    real cancat.CanInterface class.
    """
    def __init__(self, *args, **kwargs):
        # required to make canmap think the bus is working
        self.msg_count = 1

    def setCanBaud(self, *args, **kwargs):
        pass

    def placeCanBookmark(self, *args, **kwargs):
        pass

    def saveSessionToFile(self, *args, **kwargs):
        pass

    def getCanMsgCount(self):
        count = self.msg_count
        self.msg_count += 1
        return count

    def genCanMsgs(self, start=0, stop=None, arbids=None, tail=False, maxsecs=None):
        return []


class TestUDS(ScanClass):
    DIDs = {
        0x711: {
            0x0042: '\x62\x00\x42ANSWER',
        },
        0x7E0: {
            0xE010: '\x62\xE0\x10VERSION 1.2.3',
            0xF190: '\x62\xF1\x901AB123CD1EF123456',
        },
    }

    SEEDs = {
        0x711: {
            2: {
                1: 'SEED',
            }
        },
        0x7E0: {
            2: {
                1: 'FOOD',
            },
            5: {
                3: 'SEEDSEED',
            }
        },
    }

    def __init__(self, c, tx_arbid, rx_arbid=None, verbose=True, extflag=0, timeout=3.0):
        super(TestUDS, self).__init__(c, tx_arbid, rx_arbid, verbose=verbose, extflag=extflag, timeout=timeout)
        self._session = 1
        if self.tx_arbid in TestUDS.DIDs and extflag == False:
            self._dids = TestUDS.DIDs[self.tx_arbid]
        else:
            self._dids = {}

        if self.tx_arbid in TestUDS.SEEDs and extflag == False:
            self._sessions = TestUDS.SEEDs[self.tx_arbid]
        else:
            self._sessions = {}

    def ReadDID(self, did):
        if did in self._dids:
            return self._dids[did]
        elif self._dids:
            # 0x31:'RequestOutOfRange',
            raise NegativeResponseException(
                        0x31, 0x22, struct.pack('>BHB', 0x27, did, 0x31))
        else:
            # Using this timeout makes the test appear more realistic, but that 
            # is silly for testing
            #time.sleep(self.timeout)
            return None

    def WriteDID(self, did, data):
        pass

    def StartTesterPresent(self, request_response=True):
        pass

    def StopTesterPresent(self):
        pass

    def DiagnosticSessionControl(self, session):
        self._session = session
        if session in self._sessions:
            return struct.pack('>BB', 0x50, session)
        elif self._sessions:
            # 0x12:'SubFunctionNotSupported',
            raise NegativeResponseException(
                    0x12, 0x10, struct.pack('>BBB', 0x10, session, 0x12))
        else:
            # Using this timeout makes the test appear more realistic, but that 
            # is silly for testing
            #time.sleep(self.timeout)
            return None

    def RequestDownload(self, addr, data, data_format = 0x00, addr_format = 0x44):
        pass

    def RequestUpload(self, addr, length, data_format = 0x00, addr_format = 0x44):
        pass

    def readMemoryByAddress(self, address, length, lenlen=1, addrlen=4):
        pass

    def writeMemoryByAddress(self, address, data, lenlen=1, addrlen=4):
        pass

    def EcuReset(self, rst_type=0x1):
        pass

    def SecurityAccess(self, level, key):
        if self._session in self._sessions and \
                level in self._sessions[self._session]:
            seed = self._sessions[self._session][level]
            self._key_from_seed(seed, key)
            if key == seed:
                return struct.pack('>BB', 0x67, level + 1)
            elif len(key) == len(seed):
                # 0x35:'InvalidKey',
                raise NegativeResponseException(
                        0x35, 0x27, struct.pack('>BBB', 0x27, level+1, 0x35))
            else:
                # 0x13:'IncorrectMesageLengthOrInvalidFormat',
                raise NegativeResponseException(
                        0x13, 0x27, struct.pack('>BBB', 0x27, level+1, 0x13))
        elif self._sessions:
            # 0x12:'SubFunctionNotSupported',
            raise NegativeResponseException(
                    0x12, 0x27, struct.pack('>BBB', 0x27, level, 0x12))
        else:
            # Using this timeout makes the test appear more realistic, but that 
            # is silly for testing
            #time.sleep(self.timeout)
            return None